/***************************************************
 * CS182, Assignment 6: Model Merging
 * ModelMerge.java
 * Author: J.G. Makin
 * Adapted by Leon Barrett
 * Last updated: 4/02/07
 **************************************************/



import java.util.*;

/**This class holds the algorithms for dealing with the
AffixTreeNodes.  The basic operations are adding a rule, removing
a rule and finding the best merge in this tree.

The bulk of them are implemented for you.  All you need to do is to
walk through this data structure and find the best merge in this
affix tree.
*/
class AffixTree {
    AffixTreeNode root;
    double alpha;
    boolean isPrefix;
    
    
    AffixTree(boolean isP, double a) {
        isPrefix = isP;
        alpha = a;
        root = new AffixTreeNode(null, null, isPrefix);
    }
    
    /** Insert a rule into the tree, recursively.  This method must start on a node
     * below the root (i.e. we don't want to have to worry about the LHS; this method
     * only deals with the RHS.
     * 
     * @param vec
     * @param currentNode
     * @param index
     * @param r
     * @return The height of the resulting rule
     */
    private void addRuleHelper(ArrayList<String> vec, AffixTreeNode currentNode, int index, Rule r) {
        // If it's a suffix tree, we want to load the rhs in the other
        //  direction, i.e. from the end to the beginning. This line
        //  reverses the order in which we load the rhs.
        int i = isPrefix ? index : (vec.size() - index - 1);
        
        // First make sure we haven't gotten to the end of the rhs
        if(index < vec.size()) {
            // Index into the rhs vector and use this string for
            //  both keying into the hashtable of children, and
            //  assigning the affix for the new node.
            String key = vec.get(i);
            // If the current rhs element is not a child of this
            //  node, then add it.
            if((AffixTreeNode)currentNode.getChild(key) == null) {
                AffixTreeNode kid = new AffixTreeNode(key, currentNode, isPrefix);
                currentNode.addChild(kid);
                // Assign the depth, but remember that we have to
                //  add 1 for the index offset, and add another 1
                //  since the *lhs* symbol sits at depth 1.
                kid.depth = index + 2;
            }
            // Now load its child with the *next* rhs element.
            addRuleHelper(vec, currentNode.getChild(key), index + 1, r);
        } else {
            // Otherwise we've reached the end of the rule--in which case, if this is
            //  a suffixTree, we have to add in the lhs symbol as the final child.
            if(!isPrefix) {
                AffixTreeNode kid = currentNode.getChild(r.lhs);
                // Check if this node is already in the tree
                if (kid==null) {
                	kid = new AffixTreeNode(r.lhs, currentNode, isPrefix);
                    currentNode.addChild(kid);
                    kid.depth = index + 2;
                }
                
                kid.addRule(r);
            }
        }
        
        // Annotate this node with the current rule
        currentNode.addRule(r);
    }
    
    
    /** Add a rule to the tree. */
    void addRule(Rule r) {
        // Suffix merges are insensitive to the lhs symbol (though prefix merges are not),
        //  so to keep our checking routines the same for both trees, we force all rules
        //  to descend through a single first-generation node in the suffix tree (whereas
        //  in the prefix tree, they descend through their respective lhs symbols).
        String affix = isPrefix ? r.lhs : new String("DUMMY");
        
        AffixTreeNode kid = root.getChild(affix);
        // Now if this symbol is not yet in the tree, add it
        if (kid == null) {
            kid = new AffixTreeNode(affix, root, isPrefix);
            root.addChild(kid);
            kid.depth = 1;
        }
        
        // Now load up the tree, starting with the first rhs element
        addRuleHelper(r.rhs, kid, 0, r);
    }
    
    
    /** Remove all traces of a rule from the tree. */
    void removeRule(Rule r) {
        String key = new String();
        
        key = isPrefix ? r.lhs : "DUMMY";
        AffixTreeNode node = root.getChild(key);
        removeRuleHelper(node, r);
    }
    
    
    /** Recursively remove all traces of a rule from the tree. This
     * method requires starting at a level below the root (i.e. it only deals
     * with the RHS of the rule). */
    private void removeRuleHelper(AffixTreeNode node, Rule r){
        node.removeRule(r);
        if (node.getRules().size() == 0) {
            // Then this node is no longer of any use; remove it
            node.parent.removeChild(node.getAffixWord());
            // Since this severs the branch from the tree, we're
            //  done and we can just quit our recursion now
        }
		else {
            for(AffixTreeNode child : new LinkedList<AffixTreeNode>(node.getChildren())) {
                if (child.rules.contains(r)){
                    node = child;
                    removeRuleHelper(node, r);
                }
            }
            
        }
    }
    
    
    /**
	 * Run through the tree, returning only the best possible Merge for this
	 * tree (i.e. suffix or affix), where "best" means lowest changeInCost; tie
	 * goes to the shorter merge.
	 */
    Merge findBestMerge() {
    	// TODO: find the best merge in this tree
    	// note: this method isn't necessary, but it might be helpful
    	// in Grammar.findBestMerge()
    	return null;
    }
    
}
