/***************************************************
 * CS182, Assignment 6: Model Merging
 * ModelMerge.java
 * Author: J.G. Makin
 * Adapted by Leon Barrett
 * Last updated: 4/02/07
 **************************************************/


import java.util.*;

/** This class represents the grammar in progress
It keeps track of sentences that have yet to be incorporated
into the model, the current set of rules, the size of the
grammar, the total number of derivation steps for the data,
the alpha, and the total cost of the grammar, the affix trees
and other stuff most likely.*/
class Grammar {
	/** The alpha parameter for calculating cost. */
    double alpha;
    /** The cost of the grammar. */
    double cost = 0.0;
    /** The size of the grammar. */
    int size = 0;
    /** The total derivation length of the grammar. */
    int totalDerivationSteps = 0;
    /** The list of all sentences from the input file. */
    ArrayList<ArrayList<String>> sentences;
    /** A counter to keep track of what sentence we're on. */
    int currentSentenceCounter = 0;
    /** The map of rule signatures (i.e. like ["S","a","b","c"]) to
     * rules in the grammar.  This should contain all rules in the grammar.
     * So ruleMap.get(rule.signature()) == rule (if the rule is in the grammar).
     * This is the central repository of all rules in the grammar.
     */
    HashMap<ArrayList<String>,Rule> ruleMap = new HashMap<ArrayList<String>,Rule>();
    AffixTree prefixTree;
    AffixTree suffixTree;
    
    /** A counter to know what number to give to the next symbol.  The
     * new symbol should look like: "X"+newSymbolCounter.  E.g. X0, X1, etc.
     */
    int newSymbolCounter = 0;
    
    
    Grammar(double alpha, ArrayList<ArrayList<String>> sentences){
        prefixTree = new AffixTree(true, alpha);
        suffixTree = new AffixTree(false, alpha);
        this.alpha = alpha;
        this.sentences = sentences;
    }
    
    
    /**
	 * Get next sentence out of sentences, and add it to the model doing any
	 * necessary preprocessing, and adding 1 to the counter.
	 */
    void incorporateNextSentence(){
    	// TODO: add rule for next sentence
    }
    
    /** Use affix trees to find the best suffix or prefix merge. */
    Merge findBestMerge() {
    	// TODO: find best merge
    	return null;
    }
    
    /**
	 * Take a merge and change the rule set according to what the merge
	 * specifies. Update all necessary data structures and the total cost of the
	 * grammar.
	 * 
	 * @return The new rule created (not counting the modified rules.
	 */
    void performMerge(Merge m) {
    	// TODO: actually perform a merge
    }


	/** Remove a rule from the Grammar and its data structures, including cost.
	 * 
	 * @param r
	 */
	private void removeRule(Rule r) {
		// TODO: remove a rule from all data structures, updating cost.
	}


	/**
	 * Add a rule to the Grammar and its data structures, including cost. But,
	 * if the rule is already present, just increment its numTimesUsed.
	 * 
	 * @param r
	 * @return Whether true iff the rule was already in the grammar.
	 */
	private boolean addRule(Rule r) {
		// TODO: add a rule to all data structures, updating cost.
		// (don't forget to test if the rule is already present via isDuplicateRule())
		return false;
	}
	
	
	/** Test if this rule is already in our grammar by checking if it is present
	 * in our ruleMap. */
	private boolean isDuplicateRule(Rule r) {
		Rule r2 = ruleMap.get(r.signature());
		if (r2!=null)
			return true;
		return false;
	}
	
    
    /** Create new rules to replace the old ones that used to be at this node. */
    ArrayList<Rule> createNewRules(AffixTreeNode node, boolean isPrefix){
        // TODO: create the new rules for a given node and prefix
    	return null;
    }
        
    
    /** Create a new Rule for the affix represented by this node. */
    Rule createCommonAffixRule(AffixTreeNode node, boolean isPrefix){
    	// TODO: create the special rule for a merge at a node
    	return null;
    }
    
    
    /** Print out the entire rule set. */
    void printGrammar() {
    	ArrayList<Rule> rules = new ArrayList<Rule>(ruleMap.values());
    	// sort the rules
    	Collections.sort(rules, new Comparator<Rule>(){
    		public int compare(Rule r1, Rule r2) {
    			return r1.toString().compareTo(r2.toString());
    		}
    	});
    	// produce output
        System.out.println("\n Alpha: "+alpha + ";  Size: " + size +
                "; TDS: " + totalDerivationSteps + "; Cost: "
                + cost + "\n");
        for (Rule r : rules) {
            System.out.println(r);
        }
        System.out.println();
    }
    

    /**
	 * Find all nonterminals that convert directly into other nonterminals (e.g.
	 * X->Y) and make them the same
	 */
    void mergeNonTerminals() {
    	// TODO: find rules that map one non-terminal directly to another and replace
    	// one of them with the other one.
    }
    
    
}