/***************************************************
 * CS182, Assignment 6: Model Merging
 * ModelMerge.java
 * Author: J.G. Makin
 * Adapted by Leon Barrett
 * Last updated: 4/02/07
 **************************************************/



/*Class  Merge A class to encapsulate the information related to
  which rules to merge
 */
import java.util.*;

/** A representation of a rule merge. */
class Merge {
    ArrayList<Rule> rules;
    int changeInGrammarSize;
    int changeInDerivationLength;
    double changeInCost;
    AffixTreeNode mergeNode;
    boolean isPrefix;
    
    
    public Merge(AffixTreeNode node, double alpha) {
        mergeNode = node;
        this.isPrefix = node.isPrefix;
        rules = new ArrayList<Rule>(node.getRules());

        computeMergeCost(alpha);
    }
    
    
    /**
	 * Break ties between merges with the same cost. Return true if this merge
	 * should be performed before the other one.
	 */
    boolean tieBreakerMeFirst(Merge other) {
    	// break on shorter length first
    	if (this.mergeNode.affixLength() < other.mergeNode.affixLength())
    		return true;
    	if (this.mergeNode.affixLength() > other.mergeNode.affixLength())
    		return false;
    	// if that fails, break on prefix
    	if (this.isPrefix && !other.isPrefix)
    		return true;
    	if (!this.isPrefix && other.isPrefix)
    		return false;
    	// now break on alphabetical order
    	ArrayList<String> myPath = mergeNode.rootPath();
    	ArrayList<String> otherPath = other.mergeNode.rootPath();
		for (int i=0; i<myPath.size(); i++) {
			String s0 = myPath.get(i);
			String s1 = otherPath.get(i);
			int cmp = s0.compareTo(s1);
			if (cmp > 0)
				return true;
			if (cmp < 0)
				return false;
		}
    	// There's a tie, which means the two affixes are exactly the same!
		// So arbitrarily choose one.
		return true;
    }

    
    /** Compute the cost of the merge for a given alpha value. */
    void computeMergeCost(double alpha) {
    	// TODO: calculate the cost of a merge
    }
 
    
    public String toString() {
        return "**[MERGE]**\t "+changeInCost+"\t "+mergeNode.toString();
    }

}
