/***************************************************
 * CS182, Assignment 6: Model Merging
 * ModelMerge.java
 * Author: J.G. Makin
 * Last updated: 4/02/04 6:52 a.m.
 **************************************************/



/*This class represents a rule in a right linear grammar*/

import java.util.*;

/** This class represents a grammar rule.  The LHS is a nonterminal symbol
 *  (capitalized String) and the RHS is a list of symbols, of which only the
 *  last should be a nonterminal (i.e. capitalized).
 *  
 * @author leon
 *
 */
class Rule {
	/** The number of times this rule is used in the corpus. */
    int numTimesUsed = 1;
    /** The right-hand side of the rule: X -> rhs */
    ArrayList<String> rhs;
    /** The left-hand side of the rule: lhs -> a b X */
    String lhs;
    
    /** Default constructor: the rule is only used once. */
    Rule(ArrayList<String> rhs, String lhs) {
        this.rhs = new ArrayList<String>(rhs);  
        this.lhs = lhs;  
        numTimesUsed = 1;  // default to 1, though occasionally we overwrite this
    }
    
    
    public int ruleSize() {
    	return rhs.size();
    }
    
    
    /** Converts a Rule to a string for printing. */
    public String toString(){
        String retString  = "**[RULE]**\t "+lhs+ " -> ";
        for(String s : rhs){
            retString += " " + s;
        }
        retString += " ("+ruleSize() +", " + numTimesUsed + ")";
        return retString;
    }
    
    /** Get this rule's path through the prefix tree.  It starts
     * with the LHS, and continues through the RHS.
     */
    public ArrayList<String> prefixPath() {
    	ArrayList<String> path = new ArrayList<String>(ruleSize()+1);
    	path.add(lhs);
    	path.addAll(rhs);
    	return path;
    }
    
    /**
	 * Get this rule's path through the suffix tree. It starts with a dummy
	 * symbol "DUMMY", and continues backward through the RHS.
	 */
    public ArrayList<String> suffixPath() {
    	ArrayList<String> path = new ArrayList<String>(ruleSize()+1);
    	path.addAll(rhs);
    	path.add("DUMMY");
    	Collections.reverse(path);
    	return path;
    }

    /** Get the signature of this rule, for use in hashes.  It is unique for
     * each pair of LHS and RHS. */
    public ArrayList<String> signature() {
    	ArrayList<String> s = new ArrayList<String>(rhs.size()+2);
    	s.add(lhs);
    	s.add("->");
    	s.addAll(rhs);
    	return s;
    }
    
}
